@@ -19,6 +19,7 @@
  */
 
 #include "CosNaming.h"
+#include "name-support.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -115,18 +116,20 @@ Exception (CORBA_Environment * ev)
 {
   switch (ev->_major)
     {
+    case 0:	/* no exception */
+        break;
     case CORBA_SYSTEM_EXCEPTION:
-      g_printerr ("CORBA system exception.\n");
+      g_printerr ("CORBA system exception: %s.\n", CORBA_exception_id(ev));
       exit (1);
     case CORBA_USER_EXCEPTION:
       if (IS_EXCEPTION (ev, CosNaming_NamingContext_NotFound))
 	{
 	  CosNaming_NamingContext_NotFound *ex = CORBA_exception_value (ev);
-	  g_printerr ("'%s' can't be found, because '%s%s%s' is ",
-		      requested_name,
-		      ex->rest_of_name._buffer->id,
-		      strlen (ex->rest_of_name._buffer->kind) > 0 ? "." : "",
-		      ex->rest_of_name._buffer->kind);
+	  CORBA_char *name = 
+	    ORBit_CosNaming_NameComponent_to_string (ex->rest_of_name._buffer);
+	  g_printerr ("'%s' can't be found, because '%s' is ",
+		      requested_name, name);
+	  CORBA_free (name);
 	  switch (ex->why)
 	    {
 	    case CosNaming_NamingContext_missing_node:
@@ -144,13 +147,12 @@ Exception (CORBA_Environment * ev)
       else if (IS_EXCEPTION (ev, CosNaming_NamingContext_CannotProceed))
 	{
 	  CosNaming_NamingContext_CannotProceed *ex =
-	  CORBA_exception_value (ev);
+	    CORBA_exception_value (ev);
+	  CORBA_char *name = 
+	    ORBit_CosNaming_NameComponent_to_string (ex->rest_of_name._buffer);
 	  g_printerr ("The name service implementation couldn't proceed"
-		      " while processing");
-	  g_printerr ("%s%s%s.\n",
-		      ex->rest_of_name._buffer->id,
-		      strlen (ex->rest_of_name._buffer->kind) > 0 ? "." : "",
-		      ex->rest_of_name._buffer->kind);
+		      " while processing %s.\n", name);
+	  CORBA_free (name);
 	}
       else if (IS_EXCEPTION (ev, CosNaming_NamingContext_InvalidName))
 	{
@@ -173,7 +175,7 @@ Exception (CORBA_Environment * ev)
 
       exit (1);
     default:
-      break;
+      g_assert_not_reached();
     }
 }
 
@@ -183,12 +185,20 @@ assert_NamingContext (CORBA_Object object, gchar * name)
   CORBA_Environment ev;
   gboolean is_context;
 
+  if ( object==CORBA_OBJECT_NIL ) {
+      g_printerr ("'%s' is NIL (should be naming context).\n", name);
+      exit (1);
+  }
+
   CORBA_exception_init (&ev);
 
   is_context =
     CORBA_Object_is_a (object, "IDL:omg.org/CosNaming/NamingContext:1.0", &ev);
-
-  Exception (&ev);
+  if ( ev._major ) {
+      g_printerr ("Exception while verifying that '%s' is a naming context.\n",
+        name);
+      Exception (&ev);
+  }
   if (!is_context)
     {
       g_printerr ("'%s' is not a naming context.\n", name);
@@ -198,23 +208,36 @@ assert_NamingContext (CORBA_Object object, gchar * name)
 
 static CORBA_Object
 resolve (CosNaming_NamingContext context,
-	 CosNaming_Name * name,
+	 gboolean is_ext_context,
+	 CORBA_char *str,
 	 CORBA_Environment * ev)
 {
-  CORBA_Object result;
-  if (name->_length != 0)
+  CORBA_Object retval;
+  
+  if (is_ext_context)
     {
-      result = CosNaming_NamingContext_resolve (context, name, ev);
+      retval = CosNaming_NamingContextExt_resolve_str (context, str, ev);
       Exception (ev);
     }
   else
     {
-      result = CORBA_Object_duplicate (context, ev);
+      CosNaming_Name *name = ORBit_string_to_CosNaming_Name (str, ev);
       Exception (ev);
+      if (name->_length != 0)
+	{
+	  retval = CosNaming_NamingContext_resolve (context, name, ev);
+	  Exception (ev);
+	}
+      else
+	{
+	  retval = CORBA_Object_duplicate (context, ev);
+	  Exception (ev);
+	}
     }
-  return result;
+  return retval;
 }
 
+
 int
 main (int argc, char *argv[])
 {
@@ -231,6 +254,12 @@ main (int argc, char *argv[])
 
   CosNaming_Name *name = NULL;
 
+  gboolean is_ext_context;
+
+  /* Strip out ORB arguments before processing our own... */
+  CORBA_exception_init (&ev);
+  orb = CORBA_ORB_init (&argc, argv, "orbit-local-orb", &ev);
+
   if (argc < 3)
     {
       Usage (argv[0]);
@@ -257,9 +286,6 @@ main (int argc, char *argv[])
       Usage (argv[0]);
     }
 
-  CORBA_exception_init (&ev);
-
-  orb = CORBA_ORB_init (&argc, argv, "orbit-local-orb", &ev);
 
   if (strcmp (argv[1], "--gnome") == 0)
     {
@@ -284,30 +310,53 @@ main (int argc, char *argv[])
       request_cookie._buffer = principal_str;
       request_cookie._length = strlen (principal_str) + 1;
 
-      ORBit_set_default_principal (&request_cookie); 
+#if 0
+      ORBit_set_default_principal (&request_cookie);
+#endif
     }
   else if (strcmp (argv[1], "--stdin") == 0)
     fgets (objref_str, 2048, stdin);
-  else if (strncmp (argv[1], "IOR:", 4) == 0)
+  else if (strncmp (argv[1], "IOR:", 4) == 0
+       ||  strncmp (argv[1], "corbaloc:", 9) == 0 
+       ||  strncmp (argv[1], "corbaname:", 10) == 0 ) {
     strcpy (objref_str, argv[1]);
-  else
+  } else {
+    g_printerr("%s: mal-formed root argument.\n", argv[0]);
     Usage (argv[0]);
+  }
 
   g_strchug (objref_str);
   g_strchomp (objref_str);
 
-  g_assert (!(strlen (objref_str) % 2));
-
   NamingContext_obj = CORBA_ORB_string_to_object (orb, objref_str, &ev);
-  Exception (&ev);
+  if ( ev._major ) {
+    g_printerr("%s: Exception while resolving root argument.\n", argv[0]);
+    Exception (&ev);
+  }
 
   assert_NamingContext (NamingContext_obj, "root context");
+  is_ext_context = 
+    CORBA_Object_is_a (NamingContext_obj, 
+		       "IDL:omg.org/CosNaming/NamingContextExt:1.0", &ev);
+  Exception (&ev);
 
   if (argc > 3)			/* we have a name on argv[ 3 ] */
     {
       requested_name = argv[3];
-      name = ORBit_string_to_CosNaming_Name (requested_name, &ev);
-      Exception (&ev);
+      if (mode != mode_destroy && mode != mode_list && mode != mode_resolve)
+	{
+	  if (is_ext_context)
+	    {
+	      name = CosNaming_NamingContextExt_to_name (NamingContext_obj, 
+							 requested_name, &ev);
+	      Exception (&ev);
+	    }
+	  else
+	    {
+	      name = ORBit_string_to_CosNaming_Name (requested_name, &ev);
+	      Exception (&ev);
+	    }
+	}
     }
   if (argc > 4)			/* we have an object-id on argv[ 3 ] */
     {
@@ -333,7 +382,8 @@ main (int argc, char *argv[])
 					      object, &ev);
       break;
     case mode_resolve:
-      outobject = resolve (NamingContext_obj, name, &ev);
+      outobject = resolve (NamingContext_obj, is_ext_context, 
+			   requested_name, &ev);
       break;
     case mode_unbind:
       CosNaming_NamingContext_unbind (NamingContext_obj, name, &ev);
@@ -349,7 +399,8 @@ main (int argc, char *argv[])
     case mode_destroy:
       {
 	CosNaming_NamingContext remove_context = resolve (NamingContext_obj,
-							  name, &ev);
+							  is_ext_context,
+							  requested_name, &ev);
 	Exception (&ev);
 
 	assert_NamingContext (remove_context, requested_name);
@@ -365,7 +416,8 @@ main (int argc, char *argv[])
 	CosNaming_BindingList *bl;
 
 	CosNaming_NamingContext list_context = resolve (NamingContext_obj,
-							name, &ev);
+							is_ext_context,
+							requested_name, &ev);
 
 	assert_NamingContext (list_context, requested_name);
 
@@ -381,14 +433,15 @@ main (int argc, char *argv[])
 	    int i;
 	    for (i = 0; i < bl->_length; i++)
 	      {
-		CosNaming_NameComponent *name =
+		CosNaming_NameComponent *comp =
 		&bl->_buffer[i].binding_name._buffer[0];
-		printf ("%s%s%s%s\n",
-			name->id,
-			strlen (name->kind) > 0 ? "." : "",
-			name->kind,
+		CORBA_char *name = 
+		  ORBit_CosNaming_NameComponent_to_string (comp);
+
+		printf ("%s%s\n", name,
 			bl->_buffer[i].binding_type == CosNaming_ncontext ?
 			"/" : "");
+		CORBA_free (name);
 	      }
 	    CORBA_free (bl);
 	    if (CORBA_Object_is_nil (bi, &ev))
