@@ -20,22 +20,36 @@
 
 #include <glib.h>
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+
 #include "everything.h"
 #include "constants.h"
 
+#undef PROFILING
 #undef TIMING_RUN
+#undef BIG_STUBS
+#define NUM_RUNS 1000
 
-#ifndef TIMING_RUN
-#define d_print(a) g_print(a)
-#else
-#define d_print(a)
+
+#ifdef PROFILING
+#  define TIMING_RUN
 #endif
 
-void testConst() {
+#ifdef TIMING_RUN
+#  define d_print(a)
+#else
+#  define d_print(a) g_print(a)
+#endif
+
+CORBA_ORB orb;
+
+void testConst()
+{
   d_print("Testing constants...\n");
   g_assert(test_CONST_CHAR=='t');
   g_assert(test_CONST_LONG==0x12345678);
-  g_assert(test_CONST_LONGLONG==0x12345678);
+  g_assert(test_CONST_LONG_LONG==0x12345678);
   g_assert(strcmp(test_CONST_STRING,"ConstString")==0);
   /* I can never get constant floats to compare properly. */
   /* g_assert(test_CONST_FLOAT==1234.56); */
@@ -46,18 +60,23 @@ void testConst() {
 
 
 void testAttribute(test_TestFactory factory, 
-				CORBA_Environment *ev) {  
+		   CORBA_Environment *ev)
+{  
   test_BasicServer objref;
   CORBA_char *val;
   CORBA_long lval;
   d_print("Testing attributes...\n");
   objref = test_TestFactory_getBasicServer(factory,ev);
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  g_assert(objref != CORBA_OBJECT_NIL);
+  g_assert(CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
   val = test_BasicServer__get_foo(objref,ev);
   g_assert(ev->_major == CORBA_NO_EXCEPTION);  
   g_assert(strcmp(val,constants_STRING_RETN)==0);	
   CORBA_free(val);
   test_BasicServer__set_foo(objref,constants_STRING_IN,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
   lval = test_BasicServer__get_bah(objref,ev);
   g_assert(ev->_major == CORBA_NO_EXCEPTION);  
@@ -68,13 +87,23 @@ void testAttribute(test_TestFactory factory,
 }
 
 void testString(test_TestFactory factory, 
-				CORBA_Environment *ev) {
+		CORBA_Environment *ev)
+{
   test_BasicServer objref;
   const CORBA_char *in;
+  CORBA_char *ior;
   CORBA_char *inout, *out, *retn; 
+
   d_print("Testing strings...\n");
-  objref = test_TestFactory_getBasicServer(factory,ev);
+
+  ior = test_TestFactory_getBasicServerIOR (factory,ev);
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+  objref = CORBA_ORB_string_to_object (orb, ior, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  g_assert(objref != CORBA_OBJECT_NIL);
+  CORBA_free(ior);
+
   in = constants_STRING_IN;
   inout = CORBA_string_dup(constants_STRING_INOUT_IN);
   retn = test_BasicServer_opString(objref,in,&inout,&out,ev);
@@ -84,6 +113,9 @@ void testString(test_TestFactory factory,
   g_assert(strcmp(inout,constants_STRING_INOUT_OUT)==0);
   g_assert(strcmp(out,constants_STRING_OUT)==0);
   g_assert(strcmp(retn,constants_STRING_RETN)==0);	
+
+  test_BasicServer_opOneWay (objref, constants_STRING_IN, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
   
   CORBA_free(inout);
   CORBA_free(out);
@@ -94,7 +126,8 @@ void testString(test_TestFactory factory,
 
 
 void testLong(test_TestFactory factory, 
-			  CORBA_Environment *ev) {
+	      CORBA_Environment *ev)
+{
   test_BasicServer objref;
   CORBA_long inArg,inoutArg,outArg,retn;
   d_print("Testing longs...\n");
@@ -112,8 +145,29 @@ void testLong(test_TestFactory factory,
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 }
 
+void testLongLong(test_TestFactory factory, 
+		  CORBA_Environment *ev)
+{
+  test_BasicServer objref;
+  CORBA_long_long inArg,inoutArg,outArg,retn;
+  d_print("Testing long longs...\n");
+  objref = test_TestFactory_getBasicServer(factory,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  inArg = constants_LONG_LONG_IN;
+  inoutArg = constants_LONG_LONG_INOUT_IN;
+  retn = test_BasicServer_opLongLong(objref,inArg,&inoutArg,&outArg,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  g_assert(inArg == constants_LONG_LONG_IN);
+  g_assert(inoutArg == constants_LONG_LONG_INOUT_OUT);
+  g_assert(outArg == constants_LONG_LONG_OUT);
+  g_assert(retn == constants_LONG_LONG_RETN);
+  CORBA_Object_release(objref, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+}
+
 void testEnum(test_TestFactory factory, 
-			  CORBA_Environment *ev) {
+	      CORBA_Environment *ev)
+{
   test_BasicServer objref;
   test_AnEnum inArg,inoutArg,outArg,retn;
   d_print("Testing enums...\n");
@@ -132,7 +186,8 @@ void testEnum(test_TestFactory factory,
 }
 
 void testException(test_TestFactory factory, 
-				   CORBA_Environment *ev) {
+		   CORBA_Environment *ev)
+{
   test_BasicServer objref;
   test_TestException *ex;
   d_print("Testing exceptions...\n");
@@ -154,11 +209,98 @@ void testException(test_TestFactory factory,
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testFixedLengthStruct(test_TestFactory factory, 
-						   CORBA_Environment *ev) {
+gboolean
+find_tc (CORBA_sequence_CORBA_TypeCode *tcs,
+	 const char                    *repo_id)
+{
+	int i;
+
+	for (i = 0; i < tcs->_length; i++)
+		if (!strcmp (tcs->_buffer [i]->repo_id, repo_id))
+			return TRUE;
+
+	return FALSE;
+}
+
+void
+testIInterface (test_TestFactory   factory, 
+		CORBA_Environment *ev)
+{
+	test_StructServer objref;
+	CORBA_char       *type_id;
+	ORBit_IInterface *iinterface;
+	CORBA_sequence_CORBA_TypeCode *tcs;
+
+	d_print("Testing IInterface code...\n");
+	objref = test_TestFactory_getStructServer (factory,ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	/* Check nil check is working ! */
+	g_assert (CORBA_Object_is_nil  (CORBA_OBJECT_NIL, ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (!CORBA_Object_is_nil (objref, ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	/* Check non_existant is working ! */
+	g_assert (!CORBA_Object_non_existent (objref, ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	/* Ensure that we go over the wire at least once */
+	g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/StructServer:1.0", ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	/* Scripting stuff */
+
+	/* Get real type id */
+	g_assert ((type_id = ORBit_small_get_type_id (objref, ev)));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (!strcmp (type_id, "IDL:orbit/test/StructServer:1.0"));
+	CORBA_free (type_id);
+
+	/* Get interface data */
+	iinterface = ORBit_small_get_iinterface (
+		objref, "foo_bar_jelly", ev);
+	g_assert (ev->_major != CORBA_NO_EXCEPTION);
+	g_assert (iinterface == NULL);
+	g_assert (!strcmp(ev->_id, ex_ORBit_NoIInterface));
+	CORBA_exception_free (ev);
+
+	iinterface = ORBit_small_get_iinterface (
+		objref, "IDL:orbit/test/StructServer:1.0", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (iinterface != NULL);
+	g_assert (!strcmp(iinterface->tc->repo_id, "IDL:orbit/test/StructServer:1.0"));
+	CORBA_free (iinterface);
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+#define TYPELIB_NAME "./Everything_module"
+
+	if (!ORBit_small_load_typelib (TYPELIB_NAME))
+		g_warning ("Failed to load '" TYPELIB_NAME "'");
+	iinterface = ORBit_small_get_iinterface ( 
+		objref, "IDL:orbit/test/StructServer:1.0", ev);
+	g_assert(ev->_major == CORBA_NO_EXCEPTION);
+	g_assert(iinterface != NULL);
+	CORBA_free (iinterface);
+
+	tcs = ORBit_small_get_types (TYPELIB_NAME);
+	g_assert (find_tc (tcs, "IDL:orbit/test/Colour:1.0"));
+	g_assert (find_tc (tcs, "IDL:orbit/test/ArrayUnion:1.0"));
+	CORBA_free (tcs);
+}
+
+void
+testFixedLengthStruct (test_TestFactory   factory, 
+		       CORBA_Environment *ev)
+{
   test_StructServer objref;
   test_FixedLengthStruct inArg,inoutArg,outArg,retn;
-  d_print("Testing fixed length structs...\n");
+
+  d_print("Testing struct code ...\n");
   objref = test_TestFactory_getStructServer(factory,ev);
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
@@ -177,7 +319,8 @@ void testFixedLengthStruct(test_TestFactory factory,
 }
 
 void testVariableLengthStruct(test_TestFactory factory, 
-						   CORBA_Environment *ev) {
+			      CORBA_Environment *ev)
+{
   test_StructServer objref;
   test_VariableLengthStruct inArg,inoutArg, *outArg, *retn;
   d_print("Testing variable length structs...\n");
@@ -204,7 +347,8 @@ void testVariableLengthStruct(test_TestFactory factory,
 
 
 void testCompoundStruct(test_TestFactory factory, 
-						   CORBA_Environment *ev) {
+			CORBA_Environment *ev)
+{
   test_StructServer objref;
   test_CompoundStruct inArg,inoutArg, *outArg, *retn;
   d_print("Testing compound structs...\n");
@@ -230,8 +374,32 @@ void testCompoundStruct(test_TestFactory factory,
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 }
 
+void
+testStructAny (test_TestFactory   factory, 
+	       CORBA_Environment *ev)
+{
+	test_StructServer objref;
+	test_StructAny   *a;
+
+	d_print ("Testing 'any' structs...\n");
+	objref = test_TestFactory_getStructServer (factory,ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	a = test_StructServer_opStructAny (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	g_assert (!strcmp (a->a, constants_STRING_IN));
+	g_assert (*(CORBA_long *)a->b._value == constants_LONG_IN);
+  
+	CORBA_free (a);  
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
 void testUnboundedSequence(test_TestFactory factory, 
-						   CORBA_Environment *ev) {
+			   CORBA_Environment *ev)
+{
   test_SequenceServer objref;
   test_StrSeq inArg, inoutArg, *outArg, *retn;
   guint i;
@@ -241,6 +409,7 @@ void testUnboundedSequence(test_TestFactory factory,
   
   inArg._buffer = CORBA_sequence_CORBA_string_allocbuf(2);
   inArg._length = 2;
+  CORBA_sequence_set_release(&inArg, CORBA_TRUE);
 
   for (i=0;i<inArg._length;i++){
 	inArg._buffer[i] = CORBA_string_dup(constants_SEQ_STRING_IN[i]);
@@ -248,6 +417,7 @@ void testUnboundedSequence(test_TestFactory factory,
 
   inoutArg._buffer = CORBA_sequence_CORBA_string_allocbuf(2);
   inoutArg._length = 2;
+  CORBA_sequence_set_release(&inoutArg, CORBA_TRUE);
 
   for (i=0;i<inoutArg._length;i++){
 	inoutArg._buffer[i] = CORBA_string_dup(constants_SEQ_STRING_INOUT_IN[i]);
@@ -280,8 +450,10 @@ void testUnboundedSequence(test_TestFactory factory,
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testBoundedSequence(test_TestFactory factory, 
-						   CORBA_Environment *ev) {
+void
+testBoundedSequence (test_TestFactory   factory, 
+		     CORBA_Environment *ev)
+{
   test_SequenceServer objref;
   test_BoundedStructSeq inArg, inoutArg, *outArg, *retn;
   guint i;
@@ -290,17 +462,19 @@ void testBoundedSequence(test_TestFactory factory,
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
   
-  inArg._buffer = CORBA_sequence_test_CompoundStruct_allocbuf(2);
-  inArg._length = 2;
+  inArg._buffer  = CORBA_sequence_test_CompoundStruct_allocbuf(2);
+  inArg._length  = 2;
   inArg._maximum = 2;
+  CORBA_sequence_set_release(&inoutArg, CORBA_TRUE);
 
   for (i=0;i<inArg._length;i++){
 	inArg._buffer[i].a.a = CORBA_string_dup(constants_SEQ_STRING_IN[i]);
   }
 
-  inoutArg._buffer = CORBA_sequence_test_CompoundStruct_allocbuf(2);
-  inoutArg._length = 2;
+  inoutArg._buffer  = CORBA_sequence_test_CompoundStruct_allocbuf(2);
+  inoutArg._length  = 2;
   inoutArg._maximum = 2;
+  CORBA_sequence_set_release(&inoutArg, CORBA_TRUE);
 
   for (i=0;i<inoutArg._length;i++){
 	inoutArg._buffer[i].a.a = CORBA_string_dup(constants_SEQ_STRING_INOUT_IN[i]);
@@ -335,7 +509,8 @@ void testBoundedSequence(test_TestFactory factory,
 }
 
 void testFixedLengthUnion(test_TestFactory factory, 
-						  CORBA_Environment *ev) {
+			  CORBA_Environment *ev)
+{
   test_UnionServer objref;
   test_FixedLengthUnion inArg, inoutArg, outArg, retn;
   d_print("Testing fixed length unions...\n");
@@ -366,7 +541,8 @@ void testFixedLengthUnion(test_TestFactory factory,
 }
 
 void testVariableLengthUnion(test_TestFactory factory, 
-						  CORBA_Environment *ev) {
+			     CORBA_Environment *ev)
+{
   test_UnionServer objref;
   test_VariableLengthUnion inArg, inoutArg, *outArg, *retn;
   d_print("Testing variable length unions...\n");
@@ -399,7 +575,8 @@ void testVariableLengthUnion(test_TestFactory factory,
 }
 
 void testFixedLengthArray(test_TestFactory factory, 
-							 CORBA_Environment *ev) {
+			  CORBA_Environment *ev)
+{
   
   test_ArrayServer objref;
   test_LongArray inArg, inoutArg, outArg;
@@ -416,6 +593,7 @@ void testFixedLengthArray(test_TestFactory factory,
 	inoutArg[i] = constants_SEQ_LONG_INOUT_IN[i];
 
   retn = test_ArrayServer_opLongArray(objref,inArg,inoutArg,outArg,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
   for(i=0;i<constants_SEQLEN;i++)
 	g_assert(inArg[i]==constants_SEQ_LONG_IN[i]);
@@ -432,7 +610,8 @@ void testFixedLengthArray(test_TestFactory factory,
 }
 
 void testVariableLengthArray(test_TestFactory factory, 
-							 CORBA_Environment *ev) {
+			     CORBA_Environment *ev)
+{
   test_ArrayServer objref;
   test_StrArray inArg, inoutArg;
   test_StrArray_slice *outArg, *retn;
@@ -450,6 +629,7 @@ void testVariableLengthArray(test_TestFactory factory,
 	inoutArg[i] = CORBA_string_dup(constants_SEQ_STRING_INOUT_IN[i]);
 
   retn = test_ArrayServer_opStrArray(objref,inArg,inoutArg,&outArg,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
   for(i=0;i<constants_SEQLEN;i++)
 	CORBA_free (inArg[i]); 
@@ -476,8 +656,9 @@ void testVariableLengthArray(test_TestFactory factory,
 }
 
 
-void testAnyLong(test_TestFactory factory, 
-				 CORBA_Environment *ev) {
+void testAnyLong (test_TestFactory factory, 
+		  CORBA_Environment *ev)
+{
   test_AnyServer objref;
   CORBA_any inArg, inoutArg, *outArg, *retn;
   CORBA_long tmp, tmp1;
@@ -487,27 +668,28 @@ void testAnyLong(test_TestFactory factory,
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
   tmp = constants_LONG_IN;
-  inArg._type = (CORBA_TypeCode)TC_long;
+  inArg._type = (CORBA_TypeCode)TC_CORBA_long;
   inArg._value = &tmp;
   CORBA_any_set_release(&inArg, CORBA_FALSE);
 
-  inoutArg._type = (CORBA_TypeCode)TC_long;
+  inoutArg._type = (CORBA_TypeCode)TC_CORBA_long;
   tmp1 = constants_LONG_INOUT_IN;
   inoutArg._value = &tmp1;
   CORBA_any_set_release(&inoutArg, CORBA_FALSE);
     
   retn = test_AnyServer_opAnyLong(objref,&inArg,&inoutArg,&outArg,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
-  g_assert( CORBA_TypeCode_equal(inArg._type,TC_long,ev) );
+  g_assert( CORBA_TypeCode_equal(inArg._type,TC_CORBA_long,ev) );
   g_assert(*(CORBA_long*)inArg._value == constants_LONG_IN);
 
-  g_assert( CORBA_TypeCode_equal(inoutArg._type,TC_long,ev) );
+  g_assert( CORBA_TypeCode_equal(inoutArg._type,TC_CORBA_long,ev) );
   g_assert(*(CORBA_long*)inoutArg._value == constants_LONG_INOUT_OUT);
 
-  g_assert( CORBA_TypeCode_equal(outArg->_type,TC_long,ev) );
+  g_assert( CORBA_TypeCode_equal(outArg->_type,TC_CORBA_long,ev) );
   g_assert(*(CORBA_long*)outArg->_value == constants_LONG_OUT);
 
-  g_assert( CORBA_TypeCode_equal(retn->_type,TC_long,ev) );
+  g_assert( CORBA_TypeCode_equal(retn->_type,TC_CORBA_long,ev) );
   g_assert(*(CORBA_long*)retn->_value == constants_LONG_RETN);
 
   if(CORBA_any_get_release(&inArg)){
@@ -528,7 +710,8 @@ void testAnyLong(test_TestFactory factory,
 }
 
 void testAnyString(test_TestFactory factory, 
-				   CORBA_Environment *ev) {
+		   CORBA_Environment *ev)
+{
   test_AnyServer objref;
   CORBA_any inArg, inoutArg, *outArg, *retn;
 
@@ -536,26 +719,27 @@ void testAnyString(test_TestFactory factory,
   objref = test_TestFactory_getAnyServer(factory,ev);
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
-  inArg._type = (CORBA_TypeCode)TC_string;
+  inArg._type = (CORBA_TypeCode)TC_CORBA_string;
   inArg._value = &constants_STRING_IN;
   CORBA_any_set_release(&inArg, CORBA_FALSE);
 
-  inoutArg._type = (CORBA_TypeCode)TC_string;
+  inoutArg._type = (CORBA_TypeCode)TC_CORBA_string;
   inoutArg._value = &constants_STRING_INOUT_IN;
   CORBA_any_set_release(&inoutArg, CORBA_FALSE);
   
   retn = test_AnyServer_opAnyString(objref,&inArg,&inoutArg,&outArg,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
-  g_assert(CORBA_TypeCode_equal(inArg._type,TC_string,ev));
+  g_assert(CORBA_TypeCode_equal(inArg._type,TC_CORBA_string,ev));
   g_assert(strcmp(*(CORBA_char **)inArg._value,constants_STRING_IN) == 0);
 
-  g_assert(CORBA_TypeCode_equal(inoutArg._type,TC_string,ev) );
+  g_assert(CORBA_TypeCode_equal(inoutArg._type,TC_CORBA_string,ev) );
   g_assert(strcmp(*(CORBA_char **)inoutArg._value,constants_STRING_INOUT_OUT) == 0);
 
-  g_assert(CORBA_TypeCode_equal(outArg->_type,TC_string,ev) );
+  g_assert(CORBA_TypeCode_equal(outArg->_type,TC_CORBA_string,ev) );
   g_assert(strcmp(*(CORBA_char **)outArg->_value,constants_STRING_OUT) == 0);
 
-  g_assert(CORBA_TypeCode_equal(retn->_type,TC_string,ev) );
+  g_assert(CORBA_TypeCode_equal(retn->_type,TC_CORBA_string,ev) );
   g_assert(strcmp(*(CORBA_char **)retn->_value,constants_STRING_RETN) == 0);
 
   if(CORBA_any_get_release(&inArg)){
@@ -576,7 +760,8 @@ void testAnyString(test_TestFactory factory,
 }
 
 void testAnyStruct(test_TestFactory factory, 
-				   CORBA_Environment *ev) {
+		   CORBA_Environment *ev)
+{
   test_AnyServer objref;
   CORBA_any inArg, inoutArg, *outArg, *retn;
   test_VariableLengthStruct inArgStruct; 
@@ -600,6 +785,7 @@ void testAnyStruct(test_TestFactory factory,
   CORBA_any_set_release(&inoutArg, CORBA_TRUE);
 
   retn = test_AnyServer_opAnyStruct(objref,&inArg,&inoutArg,&outArg,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
   
   g_assert(CORBA_TypeCode_equal(inArg._type,TC_test_VariableLengthStruct,ev));
@@ -635,7 +821,8 @@ void testAnyStruct(test_TestFactory factory,
 }
 
 void testSequenceOfAny(test_TestFactory factory, 
-					   CORBA_Environment *ev) {
+		       CORBA_Environment *ev)
+{
   /* this test just checks the memory management for seq of any */
 
   test_AnySeq anyseq;
@@ -643,11 +830,12 @@ void testSequenceOfAny(test_TestFactory factory,
 
   d_print("Testing Sequence of Any...\n");
 
-  anyseq._buffer= CORBA_sequence_CORBA_any_allocbuf(2);
-  anyseq._length= 2;
+  anyseq._buffer  = CORBA_sequence_CORBA_any_allocbuf(2);
+  anyseq._length  = 2;
+  CORBA_sequence_set_release(&anyseq, CORBA_TRUE);
   
   for (i=0;i<anyseq._length;i++){
-	anyseq._buffer[i]._type = (CORBA_TypeCode)TC_string;
+	anyseq._buffer[i]._type = (CORBA_TypeCode)TC_CORBA_string;
 	anyseq._buffer[i]._value = &constants_STRING_IN;
 	CORBA_any_set_release(&anyseq._buffer[i], CORBA_FALSE);
   }
@@ -658,24 +846,24 @@ void testSequenceOfAny(test_TestFactory factory,
 /**
    This test tests memory management of exceptions in anys
  */
-void
-testAnyException (test_TestFactory   factory, 
-		  CORBA_Environment *ev)
+void testAnyException(test_TestFactory factory, 
+		      CORBA_Environment *ev)
 {
-	CORBA_any          *inArg; 
-	test_TestException *testex;
+  CORBA_any *inArg; 
+  test_TestException * testex;
 
-	inArg = CORBA_any__alloc ();
-	testex = test_TestException__alloc ();  
-	inArg->_type = (CORBA_TypeCode) TC_test_TestException;
-	inArg->_value = testex;
-	CORBA_any_set_release (inArg, CORBA_TRUE);
+  inArg = CORBA_any__alloc();
+  testex = test_TestException__alloc();  
+  inArg->_type = (CORBA_TypeCode) TC_test_TestException;
+  inArg->_value = testex;
+  CORBA_any_set_release(inArg,CORBA_TRUE);
 
-	CORBA_free (inArg);
+  CORBA_free(inArg);
 }
 
 void testTypeCode(test_TestFactory factory, 
-				  CORBA_Environment *ev) {
+		  CORBA_Environment *ev)
+{
   test_AnyServer objref;
   CORBA_TypeCode inArg, inoutArg, outArg, retn;
 
@@ -703,61 +891,230 @@ void testTypeCode(test_TestFactory factory,
   g_assert(ev->_major == CORBA_NO_EXCEPTION);
 }
 
-int main(int argc, char *argv[]) {
-  CORBA_Environment ev;
-  CORBA_ORB orb;
-  test_TestFactory factory;
-  int i;
-  CORBA_exception_init(&ev);
-  orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
-  g_assert(ev._major == CORBA_NO_EXCEPTION);
+void testContext(test_TestFactory factory, 
+		 CORBA_Environment *ev)
+{
+  test_ContextServer objref;
+  CORBA_Object inArg, inoutArg, outArg, retn;
+  CORBA_Context ctx;
 
+  d_print("Testing Contexts...\n");
 
-  { /* read the ior from iorfile, and swizzle to an objref*/
-	int size;
-	char ior[1024];
-	FILE *infile = fopen("iorfile","rb");
-	size = fread(ior,1,1024,infile);
-	fclose(infile);
-	ior[size] = '\0';   /* insure that string is terminated correctly */
-	factory = CORBA_ORB_string_to_object(orb, ior, &ev);
-	g_assert(ev._major == CORBA_NO_EXCEPTION);
-  }
+  objref = test_TestFactory_getContextServer(factory,ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
-#ifdef TIMING_RUN
-  for(i=0;i<1000;i++){
-#else
-  for(i=0;i<1;i++){
-#endif
-  testConst();
-  testAttribute(factory,&ev);  
-  testString(factory,&ev);
-  testLong(factory,&ev);
-  testEnum(factory,&ev);
-  testException(factory,&ev);
-  testFixedLengthStruct(factory,&ev);
-  testVariableLengthStruct(factory,&ev);
-  testCompoundStruct(factory,&ev);
-  testUnboundedSequence(factory,&ev);
-  testBoundedSequence(factory,&ev);
-  testFixedLengthUnion(factory,&ev);
-  testVariableLengthUnion(factory,&ev);
-  testFixedLengthArray(factory,&ev);
-  testVariableLengthArray(factory,&ev);
-  testAnyLong(factory,&ev);
-  testAnyString(factory,&ev);
-  testAnyStruct(factory,&ev);
-  testAnyException(factory,&ev);
-  testSequenceOfAny(factory,&ev);
-  testTypeCode(factory,&ev);
-  }
-  CORBA_Object_release(factory, &ev);
-  CORBA_Object_release((CORBA_Object)orb, &ev);
-  g_assert(ev._major == CORBA_NO_EXCEPTION);
+  inArg = inoutArg = outArg = retn = CORBA_OBJECT_NIL;
 
-#ifdef TIMING_RUN
-  g_warning ("Did '%d' iterations", i);
-#endif
+  CORBA_Context_create_child (CORBA_OBJECT_NIL, "Whatever", &ctx, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
 
-  return 0;
+  CORBA_Context_set_one_value (ctx, "foo", "foo1", ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  CORBA_Context_set_one_value (ctx, "foo", "foo2", ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  CORBA_Context_set_one_value (ctx, "bar", "baaaa", ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+  retn = test_ContextServer_opWithContext (
+	  objref, inArg, &inoutArg, &outArg, ctx, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+  CORBA_Object_release (retn, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  CORBA_Object_release (inArg, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  CORBA_Object_release (inoutArg, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+  CORBA_Object_release (outArg, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+  CORBA_Object_release ((CORBA_Object)ctx, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+  CORBA_Object_release (objref, ev);
+  g_assert(ev->_major == CORBA_NO_EXCEPTION);
+}
+
+void
+testMisc (test_TestFactory   factory, 
+	  CORBA_Environment *ev)
+{
+	CORBA_char *foo;
+
+	/* Non existant method invoke - only for remote */
+	/* FIXME: should not work in local case either ! */
+	if (!factory->adaptor_obj) {
+		/* Invoke a BasicServer method on a TestFactory */
+		foo = test_BasicServer__get_foo (factory, ev);
+		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+		g_assert (!strcmp (ev->_id, "IDL:CORBA/BAD_OPERATION:1.0"));
+		CORBA_exception_free (ev);
+	}
+
+	if (!factory->adaptor_obj) {
+		test_BasicServer objref;
+
+		objref = test_TestFactory_getBasicServer(factory,ev);
+		g_assert(ev->_major == CORBA_NO_EXCEPTION);
+		g_assert(objref != CORBA_OBJECT_NIL);
+		g_assert(CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
+		g_assert(ev->_major == CORBA_NO_EXCEPTION);
+
+		test_BasicServer_noImplement (objref, ev);
+		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+		g_assert (!strcmp (ev->_id, "IDL:CORBA/NO_IMPLEMENT:1.0"));
+		CORBA_exception_free (ev);
+
+		CORBA_Object_release (objref, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	}
+		
+	/* Check we are building full type data */
+	if (strcmp (TC_test_ObjectStruct->subtypes [0]->repo_id,
+		    "IDL:orbit/test/DerivedServer:1.0"))
+		g_warning ("Martin's bug needs fixing");
+	
+	/* Check a daft one seen in CORBA_exception_free */
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+			     ex_test_SimpleException, NULL);
+	CORBA_exception_free (ev);
+	g_assert (ev->_id == NULL);
+	g_assert (ev->_any._value == NULL);
+
+	/* TypeCode equal */
+	g_assert (CORBA_TypeCode_equal (
+		TC_CORBA_string, TC_CORBA_string, ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (!CORBA_TypeCode_equal (
+		TC_test_StrSeq, TC_test_AnotherStrSeq, ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	/* TypeCode equivalent */
+	g_assert (CORBA_TypeCode_equivalent (
+		TC_CORBA_string, TC_CORBA_string, ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (CORBA_TypeCode_equivalent (
+		TC_test_StrSeq, TC_test_AnotherStrSeq, ev));
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
+static void
+run_tests (test_TestFactory   factory,
+	   CORBA_Environment *ev,
+	   gboolean           in_proc)
+{
+	int i;
+
+#ifdef TIMING_RUN
+	for (i = 0; i < NUM_RUNS; i++) {
+#else
+	for (i = 0; i < 1; i++) {
+#endif
+		testConst ();
+		testAttribute (factory, ev);
+		testString (factory, ev);
+		testLong (factory, ev);
+		testLongLong (factory, ev);
+		testEnum (factory, ev);
+#ifndef BIG_STUBS
+		testException (factory, ev);
+#endif
+		testFixedLengthStruct (factory, ev);
+		testVariableLengthStruct (factory, ev);
+		testCompoundStruct (factory, ev);
+		testStructAny (factory, ev);
+		testUnboundedSequence (factory, ev);
+		testBoundedSequence (factory, ev);
+		testFixedLengthUnion (factory, ev);
+		testVariableLengthUnion (factory, ev);
+#ifndef BIG_STUBS
+		testFixedLengthArray (factory, ev);
+		testVariableLengthArray (factory, ev);
+		testAnyLong (factory, ev);
+		testAnyString (factory, ev);
+		testAnyStruct (factory, ev);
+		testAnyException (factory, ev);
+		testSequenceOfAny (factory, ev);
+		testTypeCode (factory, ev);
+		testContext (factory, ev);
+		testIInterface (factory, ev);
+		testMisc (factory, ev);
+#endif
+	}
+	
+#ifdef TIMING_RUN
+	g_warning ("Did '%d' iterations", i);
+#endif
+}
+
+/* Ugly- but hey */
+#define _IN_CLIENT_
+#include "server.c"
+#undef  _IN_CLIENT_
+
+int
+main (int argc, char *argv [])
+{
+	CORBA_Environment ev;
+	test_TestFactory  factory;
+
+	CORBA_exception_init (&ev);
+
+	/* Initialize threads first - so we get a threaded ORB */
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
+
+	orb = CORBA_ORB_init (&argc, argv, "orbit-local-orb", &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
+
+	free (malloc (8)); /* -lefence */
+
+	/* In Proc ... */
+#ifndef TIMING_RUN
+	{
+		factory = get_server (orb, &ev);
+		run_tests (factory, &ev, FALSE);
+		CORBA_Object_release (factory, &ev);
+		g_assert (ev._major == CORBA_NO_EXCEPTION);
+		factory = CORBA_OBJECT_NIL;
+	}
+	fprintf (stderr, "\n\n\n --- Out of proc ---\n\n\n\n");
+#endif
+
+	/* Out of Proc */
+	{ /* read the ior from iorfile, and swizzle to an objref*/
+		int   size;
+		char  ior [1024];
+		FILE *infile = fopen ("iorfile","rb");
+
+		if (!infile)
+			g_error ("Start the server before running the client");
+
+		size = fread (ior,1,1024,infile);
+		fclose (infile);
+		ior [size] = '\0';   /* insure that string is terminated correctly */
+
+		factory = CORBA_ORB_string_to_object (orb, ior, &ev);
+		g_assert (ev._major == CORBA_NO_EXCEPTION);
+
+		if (CORBA_Object_non_existent (factory, &ev))
+			g_error  ("Start the server before running the client");
+		g_assert (ev._major == CORBA_NO_EXCEPTION);
+	}
+
+	run_tests (factory, &ev, FALSE);
+
+#ifdef PROFILING
+	sleep (1000);
+#endif
+  
+	CORBA_Object_release (factory, &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
+	
+	CORBA_ORB_destroy (orb, &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
+
+	CORBA_exception_free (&ev);
+	
+	return 0;
 }
