@@ -1,29 +1,78 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
-#include <orb/orbit.h>
+#include <orbit/orbit.h>
+#include "dynany.h"
 
-#define CHECK_OK(ev)           (g_assert ((ev)->_major == CORBA_NO_EXCEPTION))
+#define CHECK_OK(ev)	g_assert ((ev)->_major == CORBA_NO_EXCEPTION)
 #define CHECK_TYPE_MISMATCH(ev) \
 	do { \
 		g_assert ((ev)->_major == CORBA_USER_EXCEPTION && \
-		          !strcmp ((ev)->_repo_id, ex_DynamicAny_DynAny_TypeMismatch)); \
+		          !strcmp ((ev)->_id, ex_DynamicAny_DynAny_TypeMismatch)); \
 		CORBA_exception_free (ev); \
 	} while (0)
 #define CHECK_INVALID_VALUE(ev) \
 	do { \
 		g_assert ((ev)->_major == CORBA_USER_EXCEPTION && \
-		          !strcmp ((ev)->_repo_id, ex_DynamicAny_DynAny_InvalidValue)); \
+		          !strcmp ((ev)->_id, ex_DynamicAny_DynAny_InvalidValue)); \
 		CORBA_exception_free (ev); \
 	} while (0)
 #define CHECK_OBJECT_NOT_EXIST(ev) \
 	do { \
 		g_assert ((ev)->_major == CORBA_SYSTEM_EXCEPTION && \
-		          !strcmp ((ev)->_repo_id, "IDL:CORBA/OBJECT_NOT_EXIST:1.0")); \
+		          !strcmp ((ev)->_id, "IDL:CORBA/OBJECT_NOT_EXIST:1.0")); \
 		CORBA_exception_free (ev); \
 	} while (0)
 
+
+static DynamicAny_DynAny
+create_basic_dyn_any (CORBA_ORB orb,
+				CORBA_TypeCode type,
+				CORBA_Environment *ev)
+{
+	DynamicAny_DynAnyFactory f;
+	DynamicAny_DynAny d;
+
+	f = (DynamicAny_DynAnyFactory)
+		CORBA_ORB_resolve_initial_references (
+			orb, "DynAnyFactory", ev);
+	CHECK_OK (ev);
+
+	d = DynamicAny_DynAnyFactory_create_dyn_any_from_type_code (
+		f, type, ev);
+	CHECK_OK (ev);
+
+	CORBA_Object_release ((CORBA_Object)f, ev);
+	CHECK_OK (ev);
+
+	return d;
+}
+
+static DynamicAny_DynAny
+create_dyn_any (CORBA_ORB orb,
+			  const CORBA_any *any,
+			  CORBA_Environment *ev)
+{
+	DynamicAny_DynAnyFactory f;
+	DynamicAny_DynAny d;
+
+	f = (DynamicAny_DynAnyFactory)
+		CORBA_ORB_resolve_initial_references (
+			orb, "DynAnyFactory", ev);
+	CHECK_OK (ev);
+
+	d = DynamicAny_DynAnyFactory_create_dyn_any (
+		f, any, ev);
+	CHECK_OK (ev);
+
+	CORBA_Object_release ((CORBA_Object)f, ev);
+	CHECK_OK (ev);
+
+	return d;
+}
+
 static int
 double_equal (double a, double b)
 {
@@ -42,7 +91,7 @@ test_long (CORBA_ORB orb, CORBA_Environment *ev)
 	CORBA_long     value;
 	CORBA_TypeCode type;
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (orb, TC_long, ev);
+	dyn_any = create_basic_dyn_any (orb, TC_CORBA_long, ev);
 	CHECK_OK (ev);
 	g_assert (dyn_any != CORBA_OBJECT_NIL);
 
@@ -76,7 +125,7 @@ test_long (CORBA_ORB orb, CORBA_Environment *ev)
 	type = DynamicAny_DynAny_type (dyn_any, ev);
 	CHECK_OK (ev);
 
-	g_assert (CORBA_TypeCode_equal (type, TC_long, ev));
+	g_assert (CORBA_TypeCode_equal (type, TC_CORBA_long, ev));
 	CHECK_OK (ev);
 
 	CORBA_Object_release ((CORBA_Object) type, ev);
@@ -91,9 +140,9 @@ test_string (CORBA_ORB orb, CORBA_Environment *ev)
 {
 	DynamicAny_DynAny dyn_any;
 	CORBA_char  *value;
-	const char  *string = "Hello World";
+	const char   string[] = "Hello World";
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (orb, TC_string, ev);
+	dyn_any = create_basic_dyn_any (orb, TC_CORBA_string, ev);
 	CHECK_OK (ev);
 	g_assert (dyn_any != CORBA_OBJECT_NIL);
 
@@ -133,9 +182,9 @@ test_copy (CORBA_ORB orb, CORBA_Environment *ev)
 	DynamicAny_DynAny dyn_any;
 	DynamicAny_DynAny dyn_any_copy;
 	CORBA_any   *any;
-	const char  *string = "Hello World2";
+	const char   string[] = "Hello World2";
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (orb, TC_string, ev);
+	dyn_any = create_basic_dyn_any (orb, TC_CORBA_string, ev);
 	CHECK_OK (ev);
 	g_assert (dyn_any != CORBA_OBJECT_NIL);
 
@@ -146,7 +195,7 @@ test_copy (CORBA_ORB orb, CORBA_Environment *ev)
 	CHECK_OK (ev);
 	g_assert (any != NULL);
 
-	dyn_any_copy = CORBA_ORB_create_dyn_any (orb, any, ev);
+	dyn_any_copy = create_dyn_any (orb, any, ev);
 	CHECK_OK (ev);
 	g_assert (dyn_any_copy != NULL);
 	CORBA_free (any);
@@ -155,8 +204,8 @@ test_copy (CORBA_ORB orb, CORBA_Environment *ev)
 	CHECK_OK (ev);
 
 	{ /* Knock up an integer any */
-		DynamicAny_DynAny int_any = CORBA_ORB_create_basic_dyn_any (
-			orb, TC_long, ev);
+		DynamicAny_DynAny int_any = create_basic_dyn_any (
+			orb, TC_CORBA_long, ev);
 		CHECK_OK (ev);
 
 		DynamicAny_DynAny_insert_long (int_any, 57, ev);
@@ -186,39 +235,23 @@ test_copy (CORBA_ORB orb, CORBA_Environment *ev)
 	CHECK_OK (ev);
 }
 
-static const CORBA_TypeCode
-octet_subtypes_array [] = {
-	TC_octet
-};
-
-static const struct CORBA_TypeCode_struct
-TC_CORBA_sequence_CORBA_octet_struct = {
-      {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
-       ORBIT_PSEUDO_TYPECODE},
-
-      CORBA_tk_sequence, NULL, NULL,
-      0, 1,
-      NULL,
-      (CORBA_TypeCode *) octet_subtypes_array,
-      NULL,
-      CORBA_OBJECT_NIL, 0, -1, 0, 0
-};
-
 static void
 test_sequence (CORBA_ORB orb, CORBA_Environment *ev)
 {
-	DynamicAny_DynAny dyn_any;
+	DynamicAny_DynAny      dyn_any;
+	DynamicAny_DynSequence dyn_seq;
 	int          i, len;
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (
-		orb, (CORBA_TypeCode) &TC_CORBA_sequence_CORBA_octet_struct, ev);
+	dyn_any = create_basic_dyn_any (
+		orb, TC_CORBA_sequence_CORBA_octet, ev);
+	dyn_seq = (DynamicAny_DynSequence) dyn_any;
 	CHECK_OK (ev);
 	g_assert (dyn_any != CORBA_OBJECT_NIL);
 
 	DynamicAny_DynAny_insert_long (dyn_any, 5, ev);
 	CHECK_TYPE_MISMATCH (ev);
 
-	DynamicAny_DynSequence_set_length (dyn_any, 100, ev);
+	DynamicAny_DynSequence_set_length (dyn_seq, 100, ev);
 	CHECK_OK (ev);
 
 	for (i = 0; i < 100; i++) {
@@ -232,7 +265,7 @@ test_sequence (CORBA_ORB orb, CORBA_Environment *ev)
 	CHECK_OK (ev);
 	g_assert (len == 100);
 
-	len = DynamicAny_DynSequence_get_length (dyn_any, ev);
+	len = DynamicAny_DynSequence_get_length (dyn_seq, ev);
 	CHECK_OK (ev);
 	g_assert (len == 100);
 
@@ -240,10 +273,10 @@ test_sequence (CORBA_ORB orb, CORBA_Environment *ev)
 	DynamicAny_DynAny_seek (dyn_any, -1, ev);
 	CHECK_OK (ev);
 
-	DynamicAny_DynSequence_set_length (dyn_any, 150, ev);
+	DynamicAny_DynSequence_set_length (dyn_seq, 150, ev);
 	CHECK_OK (ev);
 
-	len = DynamicAny_DynSequence_get_length (dyn_any, ev);
+	len = DynamicAny_DynSequence_get_length (dyn_seq, ev);
 	CHECK_OK (ev);
 	g_assert (len == 150);
 
@@ -255,7 +288,7 @@ test_sequence (CORBA_ORB orb, CORBA_Environment *ev)
 
 	g_assert (i == 137);
 
-	DynamicAny_DynSequence_set_length (dyn_any, 200, ev);
+	DynamicAny_DynSequence_set_length (dyn_seq, 200, ev);
 	CHECK_OK (ev);
 
 	i = DynamicAny_DynAny_get_octet (dyn_any, ev);
@@ -263,17 +296,17 @@ test_sequence (CORBA_ORB orb, CORBA_Environment *ev)
 
 	g_assert (i == 137);
 
-	len = DynamicAny_DynSequence_get_length (dyn_any, ev);
+	len = DynamicAny_DynSequence_get_length (dyn_seq, ev);
 	CHECK_OK (ev);
 	g_assert (len == 200);
 
 	{
-		DynamicAny_DynAny_AnySeq *seq;
+		DynamicAny_AnySeq *seq;
 
-		seq = DynamicAny_DynSequence_get_elements (dyn_any, ev);
+		seq = DynamicAny_DynSequence_get_elements (dyn_seq, ev);
 		CHECK_OK (ev);
 
-		DynamicAny_DynSequence_set_elements (dyn_any, seq, ev);
+		DynamicAny_DynSequence_set_elements (dyn_seq, seq, ev);
 		CHECK_OK (ev);
 
 		CORBA_free (seq);
@@ -283,24 +316,6 @@ test_sequence (CORBA_ORB orb, CORBA_Environment *ev)
 	CHECK_OK (ev);
 }
 
-static const CORBA_TypeCode
-array_subtypes_array [] = {
-	TC_double
-};
-
-static const struct CORBA_TypeCode_struct
-TC_my_array_struct = {
-      {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
-       ORBIT_PSEUDO_TYPECODE},
-
-      CORBA_tk_array, NULL, NULL,
-      100, 1,
-      NULL,
-      (CORBA_TypeCode *) array_subtypes_array,
-      NULL,
-      CORBA_OBJECT_NIL, 0, -1, 0, 0
-};
-
 static void
 test_array (CORBA_ORB orb, CORBA_Environment *ev)
 {
@@ -308,8 +323,8 @@ test_array (CORBA_ORB orb, CORBA_Environment *ev)
 	CORBA_double d;
 	int          i;
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (
-		orb, (CORBA_TypeCode) &TC_my_array_struct, ev);
+	dyn_any = create_basic_dyn_any (
+		orb, TC_Test_MyArray, ev);
 	CHECK_OK (ev);
 	g_assert (dyn_any != CORBA_OBJECT_NIL);
 
@@ -355,58 +370,45 @@ enum_subnames_array [] = {
 	"HERRING"
 };
 
-const struct CORBA_TypeCode_struct
-TC_org_fish_packers_Fishy_struct = {
-   
-	{{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
-	 ORBIT_PSEUDO_TYPECODE},
-	
-	CORBA_tk_enum, "Fishy", "IDL:org/fish/packers/Fishy:1.0",
-	0, 3,
-	(const char **) enum_subnames_array,
-	NULL,
-	NULL,
-	CORBA_OBJECT_NIL, 0, -1, 0, 0
-};
-
 static void
 test_enum (CORBA_ORB orb, CORBA_Environment *ev)
 {
-	DynamicAny_DynAny dyn_any;
-	int          i;
+	int i;
+	DynamicAny_DynEnum dyn_enum;
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (
-		orb, (CORBA_TypeCode) &TC_org_fish_packers_Fishy_struct, ev);
+	dyn_enum = (DynamicAny_DynEnum)
+		create_basic_dyn_any (
+			orb, TC_Test_Fishy, ev);
 	CHECK_OK (ev);
-	g_assert (dyn_any != CORBA_OBJECT_NIL);
+	g_assert (dyn_enum != CORBA_OBJECT_NIL);
 
-	i = DynamicAny_DynEnum_get_as_ulong (dyn_any, ev);
+	i = DynamicAny_DynEnum_get_as_ulong (dyn_enum, ev);
 	CHECK_OK (ev);
 	g_assert (i == 0);
 
 	for (i = 0; i < sizeof (enum_subnames_array) / sizeof (const char *); i++) {
 
-		DynamicAny_DynEnum_set_as_string (dyn_any, enum_subnames_array [i], ev);
+		DynamicAny_DynEnum_set_as_string (dyn_enum, enum_subnames_array [i], ev);
 		CHECK_OK (ev);
 
-		g_assert (DynamicAny_DynEnum_get_as_ulong (dyn_any, ev) == i);
+		g_assert (DynamicAny_DynEnum_get_as_ulong (dyn_enum, ev) == i);
 		CHECK_OK (ev);
 	}
 
 	for (i = 0; i < sizeof (enum_subnames_array) / sizeof (const char *); i++) {
 		CORBA_char  *str;
 
-		DynamicAny_DynEnum_set_as_ulong (dyn_any, i, ev);
+		DynamicAny_DynEnum_set_as_ulong (dyn_enum, i, ev);
 		CHECK_OK (ev);
 
-		str = DynamicAny_DynEnum_get_as_string (dyn_any, ev);
+		str = DynamicAny_DynEnum_get_as_string (dyn_enum, ev);
 		CHECK_OK (ev);
 
 		g_assert (!strcmp (str, enum_subnames_array [i]));
 		CORBA_free (str);
 	}
 
-	CORBA_Object_release ((CORBA_Object) dyn_any, ev);
+	CORBA_Object_release ((CORBA_Object) dyn_enum, ev);
 	CHECK_OK (ev);
 }
 
@@ -419,79 +421,32 @@ union_subnames_array [] = {
 	"ibid"
 };
 
-static CORBA_unsigned_long 
-union_sublabel_values_array [] = {
-	0, 1, 2, 3, 4
-};
-
-static const CORBA_TypeCode
+static CORBA_TypeCode
 union_subtypes_array [] = {
-	TC_long,
-	TC_double,
-	TC_string,
-	(CORBA_TypeCode) & TC_org_fish_packers_Fishy_struct,
-	(CORBA_TypeCode) & TC_CORBA_sequence_CORBA_octet_struct
-};
-
-static const CORBA_any
-union_sublabels_array [] = {
-	{ (CORBA_TypeCode) TC_CORBA_unsigned_long, 
-	  &union_sublabel_values_array [0], CORBA_FALSE },
-	{ (CORBA_TypeCode) TC_CORBA_unsigned_long,
-	  &union_sublabel_values_array [1], CORBA_FALSE },
-	{ (CORBA_TypeCode) TC_CORBA_unsigned_long,
-	  &union_sublabel_values_array [2], CORBA_TRUE  },
-	{ (CORBA_TypeCode) TC_CORBA_unsigned_long,
-	  &union_sublabel_values_array [3], CORBA_FALSE },
-	{ (CORBA_TypeCode) TC_CORBA_unsigned_long,
-	  &union_sublabel_values_array [4], CORBA_FALSE },
-};
-
-const struct CORBA_TypeCode_struct
-TC_England_Unions_UnionValue_struct = {
-   
-      {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
-       ORBIT_PSEUDO_TYPECODE},
-
-      CORBA_tk_union, "UnionValue", "IDL:England/Unions/UnionValue:1.0",
-      0, 5,
-      (const char **)    union_subnames_array,
-      (CORBA_TypeCode *) union_subtypes_array,
-      (CORBA_any *)      union_sublabels_array,
-      (CORBA_TypeCode)   TC_CORBA_unsigned_long,
-      0, -1, 0, 0
-};
-
-const struct CORBA_TypeCode_struct
-TC_England_Unions_Struct_struct = {
-   
-      {{(ORBit_RootObject_Interface *) & ORBit_TypeCode_epv, TRUE, -1},
-       ORBIT_PSEUDO_TYPECODE},
-
-      CORBA_tk_struct, "Struct", "IDL:England/Unions/Struct:1.0",
-      0, 5,
-      (const char **)    union_subnames_array,
-      (CORBA_TypeCode *) union_subtypes_array,
-      NULL,
-      CORBA_OBJECT_NIL, 0, -1, 0, 0
+	TC_CORBA_long,
+	TC_CORBA_double,
+	TC_CORBA_string,
+	TC_Test_Fishy,
+	TC_Test_OSeq
 };
 
 static void
 test_union (CORBA_ORB orb, CORBA_Environment *ev)
 {
-	DynamicAny_DynAny dyn_any;
-	CORBA_TCKind kind;
+	DynamicAny_DynUnion dyn_union;
+	CORBA_TCKind        kind;
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (
-		orb, (CORBA_TypeCode) &TC_England_Unions_UnionValue_struct, ev);
+	dyn_union = (DynamicAny_DynUnion)
+		create_basic_dyn_any (
+			orb, TC_Test_English, ev);
 	CHECK_OK (ev);
-	g_assert (dyn_any != CORBA_OBJECT_NIL);
+	g_assert (dyn_union != CORBA_OBJECT_NIL);
 
-	kind = DynamicAny_DynUnion_discriminator_kind (dyn_any, ev);
+	kind = DynamicAny_DynUnion_discriminator_kind (dyn_union, ev);
 	CHECK_OK (ev);
 	g_assert (kind == CORBA_tk_ulong);
 
-	CORBA_Object_release ((CORBA_Object) dyn_any, ev);
+	CORBA_Object_release ((CORBA_Object) dyn_union, ev);
 	CHECK_OK (ev);
 }
 
@@ -499,14 +454,18 @@ static void
 test_struct (CORBA_ORB orb, CORBA_Environment *ev)
 {
 	DynamicAny_DynAny dyn_any, sub_any;
+	DynamicAny_DynStruct dyn_struct;
+	DynamicAny_NameValuePairSeq *members;
+	/* DynamicAny_NameDynAnyPairSeq *dyn_members; */
 	CORBA_TCKind kind;
 	CORBA_char  *str;
 	CORBA_double dv = 1.23;
 	const char  *test_str = "one is not amused";
 	int i;
 
-	dyn_any = CORBA_ORB_create_basic_dyn_any (
-		orb, (CORBA_TypeCode) &TC_England_Unions_Struct_struct, ev);
+	dyn_any = create_basic_dyn_any (
+		orb, TC_Test_Unions, ev);
+	dyn_struct = (DynamicAny_DynStruct) dyn_any;
 	CHECK_OK (ev);
 	g_assert (dyn_any != CORBA_OBJECT_NIL);
 
@@ -517,12 +476,12 @@ test_struct (CORBA_ORB orb, CORBA_Environment *ev)
 		DynamicAny_DynStruct_seek (dyn_any, i, ev);
 		CHECK_OK (ev);
 
-		kind = DynamicAny_DynStruct_current_member_kind (dyn_any, ev);
+		kind = DynamicAny_DynStruct_current_member_kind (dyn_struct, ev);
 		CHECK_OK (ev);
 
 		g_assert (union_subtypes_array [i]->kind == kind);
 
-		str = DynamicAny_DynStruct_current_member_name (dyn_any, ev);
+		str = DynamicAny_DynStruct_current_member_name (dyn_struct, ev);
 		CHECK_OK (ev);
 
 		g_assert (!strcmp (union_subnames_array [i], str));
@@ -572,6 +531,26 @@ test_struct (CORBA_ORB orb, CORBA_Environment *ev)
 	g_assert (DynamicAny_DynAny_next (dyn_any, ev));
 	CHECK_OK (ev);
 
+	members = DynamicAny_DynStruct_get_members (dyn_struct, ev);
+	g_assert (members != NULL);
+	CHECK_OK (ev);
+
+	DynamicAny_DynStruct_set_members (dyn_struct, members, ev);
+	CHECK_OK (ev);
+
+	CORBA_free (members);
+
+/*	it's not clear how best to deal with this:
+
+	dyn_members = DynamicAny_DynStruct_get_members_as_dyn_any (dyn_any, ev);
+	g_assert (dyn_members != NULL);
+	CHECK_OK (ev);
+
+	DynamicAny_DynStruct_set_members_as_dyn_any (dyn_any, dyn_members, ev);
+	CHECK_OK (ev);
+
+	CORBA_free (dyn_members);*/
+
 	CORBA_Object_release ((CORBA_Object) dyn_any, ev);
 	CHECK_OK (ev);
 
@@ -582,6 +561,8 @@ test_struct (CORBA_ORB orb, CORBA_Environment *ev)
 	CHECK_OK (ev);
 }
 
+#define d_printf printf
+
 int
 main (int argc, char *argv[])
 {
@@ -597,19 +578,35 @@ main (int argc, char *argv[])
 	 *  Since the API is entirely macro generated
 	 * we only need to test a few cases.
 	 */
+	d_printf      ("Testing basic DynAny ...\n");
+	d_printf      (" + long ops ...\n");
 	test_long     (orb, &ev);
+
+	d_printf      (" + string ops ...\n");
 	test_string   (orb, &ev);
+
+	d_printf      (" + copying ...\n");
 	test_copy     (orb, &ev);
+
+	d_printf      ("Testing DynSequence ...\n");
 	test_sequence (orb, &ev);
+
+	d_printf      ("Testing DynEnum ...\n");
 	test_enum     (orb, &ev);
+
+	d_printf      ("Testing DynUnion ...\n");
 	test_union    (orb, &ev);
+
+	d_printf      ("Testing DynArray...\n");
 	test_array    (orb, &ev);
+
+	d_printf      ("Testing DynStruct...\n");
 	test_struct   (orb, &ev);
 
 	CORBA_Object_release ((CORBA_Object) orb, &ev);
 	CHECK_OK (&ev);
   
-	printf ("all DynAny tests passed ok.\n");
+	d_printf ("all DynAny tests passed ok.\n");
 
 	return 0;
 }
